@@ -16,7 +16,7 @@
 
 //! Set of types for supporting [CBOR Object Signing and Encryption (COSE)][COSE].
 //!
-//! Builds on the [`sk-cbor`](https://docs.rs/sk-cbor) crate for underlying [CBOR][CBOR] support.
+//! Builds on the [`ciborium`](https://docs.rs/ciborium) crate for underlying [CBOR][CBOR] support.
 //!
 //! ## Usage
 //!
@@ -66,6 +66,8 @@
 //! // At the receiving end, deserialize the bytes back to a `CoseSign1` object.
 //! let mut sign1 = coset::CoseSign1::from_slice(&sign1_data).unwrap();
 //!
+//! // At this point, real code would validate the protected headers.
+//!
 //! // Check the signature, which needs to have the same `aad` provided, by
 //! // providing a closure that can do the verify operation.
 //! let result = sign1.verify_signature(aad, |sig, data| verifier.verify(sig, data));
@@ -84,7 +86,8 @@
 //!     .is_err());
 //!
 //! // Changing a protected header invalidates the signature.
-//! sign1.protected.content_type = Some(coset::ContentType::Text("text/plain".to_owned()));
+//! sign1.protected.original_data = None;
+//! sign1.protected.header.content_type = Some(coset::ContentType::Text("text/plain".to_owned()));
 //! assert!(sign1
 //!     .verify_signature(aad, |sig, data| verifier.verify(sig, data))
 //!     .is_err());
@@ -94,11 +97,11 @@
 //! [CBOR]: https://tools.ietf.org/html/rfc7049
 
 #![no_std]
-#![deny(broken_intra_doc_links)]
+#![deny(rustdoc::broken_intra_doc_links)]
 extern crate alloc;
 
-/// Re-export of the `sk-cbor` crate used for underlying CBOR encoding.
-pub use sk_cbor as cbor;
+/// Re-export of the `ciborium` crate used for underlying CBOR encoding.
+pub use ciborium as cbor;
 
 #[macro_use]
 pub(crate) mod util;
