@@ -15,26 +15,23 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 use super::*;
-use crate::{
-    cbor::values::{SimpleValue, Value},
-    util::expect_err,
-};
+use crate::{cbor::value::Value, util::expect_err};
 use alloc::{borrow::ToOwned, boxed::Box, vec};
 
 #[test]
 fn test_cbor_type_error() {
     let cases = vec![
-        (Value::Simple(SimpleValue::NullValue), "null"),
-        (Value::Simple(SimpleValue::TrueValue), "true"),
-        (Value::Simple(SimpleValue::FalseValue), "false"),
-        (Value::Simple(SimpleValue::Undefined), "undefined"),
-        (Value::Unsigned(128), "uint"),
-        (Value::Negative(-1), "nint"),
-        (Value::ByteString(vec![1, 2]), "bstr"),
-        (Value::TextString("string".to_owned()), "tstr"),
-        (Value::Array(vec![Value::Unsigned(0)]), "array"),
+        (Value::Null, "nul"),
+        (Value::Bool(true), "bool"),
+        (Value::Bool(false), "bool"),
+        (Value::from(128), "int"),
+        (Value::from(-1), "int"),
+        (Value::Bytes(vec![1, 2]), "bstr"),
+        (Value::Text("string".to_owned()), "tstr"),
+        (Value::Array(vec![Value::from(0)]), "array"),
         (Value::Map(vec![]), "map"),
-        (Value::Tag(1, Box::new(Value::Unsigned(0))), "tag"),
+        (Value::Tag(1, Box::new(Value::from(0))), "tag"),
+        (Value::Float(1.054571817), "float"),
     ];
     for (val, want) in cases {
         let e = cbor_type_error::<()>(&val, "a");
